"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.remove = exports.list = exports.write = exports.read = exports.drain = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
async function drain(stream) {
    return await new Promise((resolve, reject) => {
        const chunks = [];
        stream.on('data', (chunk) => chunks.push(chunk));
        stream.on('error', reject);
        stream.on('end', () => resolve(Buffer.concat(chunks)));
    });
}
exports.drain = drain;
async function read(path, { range } = {}) {
    const [begin = 0, end = Infinity] = range || [];
    if (begin < 0 || end < 0 || begin > end)
        throw new Error('Invalid range');
    await fs_1.default.promises.access(path);
    const stream = fs_1.default.createReadStream(path, {
        start: begin,
        end: end - 1,
        autoClose: true,
    });
    return drain(stream);
}
exports.read = read;
async function write(path, data) {
    return fs_1.default.promises.writeFile(path, data);
}
exports.write = write;
/*
export async function createReadStream(
  path: string,
  { range }: Types.ReadOptions = {}
): Promise<ReadableStream<Uint8Array>> {
  const [begin = 0, end = Infinity] = range || []
  if (begin < 0 || end < 0 || begin > end) throw new Error('Invalid range')

  await fs.promises.access(path)
  const fstream = fs.createReadStream(path, {
    start: begin,
    end: end - 1,
    autoClose: true,
  })

  class FsTransformer extends TransformStream<Buffer, Uint8Array> {
    constructor(stream: fs.ReadStream) {
      super({
        start() {},
        flush() {},
        async transform(chunk, controller) {
          controller.enqueue(new Uint8Array(chunk.buffer))
        },
      })
    }
  }

  const transformer = new FsTransformer(fstream)
  return transformer.readable
}

export async function writeStream(path: string, data: NodeJS.ReadableStream) {
  await new Promise((resolve, reject) => {
    const w = fs.createWriteStream(path)
    w.on('close', resolve)
    w.on('error', reject)
    data.pipe(w)
  })
}
*/
async function list(path) {
    // We'll keep the behavior here simple for now and skip oddities like FIFOs,
    // sockets, block devices, symlinks, etc.  We could follow symlinks but for
    // now we'll skip those too.
    const list = (await fs_1.default.promises.readdir(path, { withFileTypes: true }))
        .filter((dirent) => dirent.isFile() || dirent.isDirectory())
        .map((dirent) => ({
        path: dirent.name,
        type: dirent.isDirectory() ? 'directory' : 'file',
    }));
    return Promise.all(list.map(async (entry) => {
        if (entry.type === 'directory')
            return entry;
        const { size } = await fs_1.default.promises.stat(path_1.join(path, entry.path));
        return { ...entry, size };
    }));
}
exports.list = list;
async function remove(path) {
    await fs_1.default.promises.unlink(path);
}
exports.remove = remove;
function create() {
    return {
        read,
        write,
        /*
        createReadStream,
        writeStream,
        */
        list,
        remove,
    };
}
exports.create = create;
