"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.remove = exports.list = exports.copyFile = exports.write = exports.readJson = exports.readString = exports.read = exports.getProtocolOrDefault = exports.create = exports.Options = void 0;
const protopath_1 = require("protopath");
const Dropbox = __importStar(require("./dropbox"));
const File = __importStar(require("./file"));
const Http = __importStar(require("./http"));
const S3 = __importStar(require("./s3"));
const options_1 = require("./options");
Object.defineProperty(exports, "Options", { enumerable: true, get: function () { return options_1.Options; } });
function createPartial(protocol, options) {
    switch (protocol) {
        case 's3': {
            if (options && !S3.isOptions(options)) {
                throw new Error('Invalid S3 options');
            }
            return S3.create(options);
        }
        case 'dropbox':
        case 'dbx': {
            if (!Dropbox.isOptions(options)) {
                throw new Error('Invalid dropbox options');
            }
            return Dropbox.create(options);
        }
        case 'http':
        case 'https':
            return Http.create(protocol);
        case '':
        case 'file':
            return File.create();
        default:
            throw new Error('Invalid protocol');
    }
}
const methods = [
    'read',
    'write',
    'createReadStream',
    'writeStream',
    'list',
    'remove',
];
function createThrower(protocol) {
    return methods.reduce((thrower, method) => ({
        ...thrower,
        [method]: async function () {
            throw new Error(`${protocol}: ${method} not supported`);
        },
    }), {});
}
function create(protocol, options) {
    const thrower = createThrower(protocol);
    const partial = createPartial(protocol, options);
    return { ...thrower, ...partial };
}
exports.create = create;
function getProtocolOrDefault(path) {
    return protopath_1.getProtocol(path) || 'file';
}
exports.getProtocolOrDefault = getProtocolOrDefault;
async function read(path, options) {
    const s = create(getProtocolOrDefault(path));
    return s.read(protopath_1.stripProtocol(path), options);
}
exports.read = read;
async function readString(path, options) {
    return (await read(path, options)).toLocaleString();
}
exports.readString = readString;
async function readJson(path, options) {
    return JSON.parse(await readString(path, options));
}
exports.readJson = readJson;
async function write(path, data) {
    const s = create(getProtocolOrDefault(path));
    return s.write(protopath_1.stripProtocol(path), data);
}
exports.write = write;
/*
export async function createReadStream(
  path: string,
  options?: Types.ReadOptions
) {
  const s = create(getProtocolOrDefault(path))
  return s.createReadStream(stripProtocol(path), options)
}
export async function writeStream(
  path: string,
  data: ReadableStream<Uint8Array>
) {
  const s = create(getProtocolOrDefault(path))
  return s.writeStream(stripProtocol(path), data)
}
*/
async function copyFile(input, output) {
    return write(output, await read(input));
    // return writeStream(output, await createReadStream(input))
}
exports.copyFile = copyFile;
async function list(dir, recursive = false) {
    const s = create(getProtocolOrDefault(dir));
    const contents = await s.list(protopath_1.stripProtocol(dir));
    if (!recursive)
        return contents;
    return (await Promise.all(contents.map(async (item) => {
        if (item.type === 'file')
            return [item];
        // Note that this `list` call returns items relative to our
        // subdirectory, while we want them relative to our listing root, so
        // we'll have to join them properly below.
        return (await list(protopath_1.join(dir, item.path), true)).map((v) => ({
            ...v,
            path: protopath_1.join(item.path, v.path),
        }));
    }))).reduce((list, curr) => [...list, ...curr], []);
}
exports.list = list;
async function remove(path) {
    const s = create(getProtocolOrDefault(path));
    return s.remove(protopath_1.stripProtocol(path));
}
exports.remove = remove;
