"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.readText = exports.readJson = exports.read = exports.ResponseError = void 0;
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const range_1 = require("./range");
class ResponseError extends Error {
    constructor(code, text, body) {
        super(text);
        this.code = code;
        this.body = body;
    }
}
exports.ResponseError = ResponseError;
async function run(url, options) {
    const response = await cross_fetch_1.default(url, options);
    const { ok, status, statusText } = response;
    if (!ok) {
        const type = response.headers.get('content-type');
        const body = type && type.startsWith('application/json')
            ? await response.json()
            : await response.text();
        throw new ResponseError(status, statusText, body);
    }
    return response;
}
async function get(url, { range, ...options } = {}) {
    if (range) {
        options.headers = { ...options.headers, Range: range_1.Range.toHeaderValue(range) };
    }
    return run(url, options);
}
async function read(path, options) {
    const response = await get(path, options);
    return response.arrayBuffer();
}
exports.read = read;
/*
export async function createReadStream(
  path: string,
  options?: Options
): Promise<ReadableStream<Uint8Array>> {
  const response = await get(path, options)
  const body = response.body
  if (!body) throw new Error('Missing response body')
  return body
}
*/
async function readJson(path, options) {
    return (await get(path, options)).json();
}
exports.readJson = readJson;
async function readText(path, options) {
    return (await get(path, options)).text();
}
exports.readText = readText;
function create(protocol) {
    function prefix(path) {
        return `${protocol}://${path}`;
    }
    async function protoRead(path, options) {
        return read(prefix(path), options);
    }
    /*
    async function protoCreateReadStream(path: string, options?: Options) {
      return createReadStream(prefix(path), options)
    }
    */
    return {
        read: protoRead,
    };
}
exports.create = create;
